/*
 * Zombie - WidgetControl overlay manager for Qt
 *
 * Author: Jérémie Gilliot <jeremie.gilliot@inria.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ZOMBIE_H
#define ZOMBIE_H

#include <QApplication>
#include <QWidget>
#include <QGraphicsView>
#include <QEvent>
#include <QTouchEvent>

class GhostView;
class FeedbackCreation;
class QTuio;
class AbstractWidget;

#if defined(TELETOUCH_LIBRARY)
#  define TELETOUCH_DLLSPEC Q_DECL_EXPORT
#else
#  define TELETOUCH_DLLSPEC Q_DECL_IMPORT
#endif


/** An Zombie widget shows all the QTouchPoints received from its
    parent or from any of the parent's descendants. An Zombie widget
    always requires a parent QWidget. */
class TELETOUCH_DLLSPEC Zombie : public QWidget {
    Q_OBJECT

private:
    QColor colors[10];
    int colorIndex;

    QGraphicsScene *theParentScene;
    QGraphicsView *theParentView;
    QWidget *theParentWidget;

    GhostView *ghostView;

    FeedbackCreation *ffbkCreate;
    int dissociateControlId;
    int dissociateCursorId;

    QTuio *qTUIO;

    float px2mmScreen, px2mmTouchpad;
    QSizeF sizeTouchpad;

    void createRipple(QWidget *widget, const QTouchEvent::TouchPoint &touchPoint);
    bool eventFilter(QObject *obj, QEvent *event);

public:
    Zombie(QWidget *parent);
    ~Zombie();

    GhostView *getGhostView(){ return ghostView; }

public slots:
    AbstractWidget *createGhost(QPointF widgetPos, QPointF cursorPos);
    void cancellingCreation();
} ;

#endif
