#ifndef ABSTRACTWIDGET_H
#define ABSTRACTWIDGET_H

#include <QGraphicsObject>
#include <QTouchEvent>
#include <QTime>
#include <QTimer>

#include "ghostview.h"
#include "cursor.h"
#include "link.h"
#include "SpeedListener.h"

class GhostView;
class Cursor;

class AbstractWidget : public QGraphicsObject, public SpeedListener
{
    Q_OBJECT

protected:
    GhostView *ghostview;

    QMap <int, Cursor*> cursors;
    QMap<int, int> fingerControlandCursorId;  //Permet d'acceder a la position des doigts disposés sur le widget (hors bouton et RSbubbles) et au curseur lie
    QMap<int, int> touchIdTOcurselId;

    QMap<int, QTouchEvent::TouchPoint> listPtsOnWidget;  //used for experience monitoring

    float x_width, y_height;

    int memoryLeftClick, memoryRightClick;

    QMap<int, QTime*> times;

    QString name;
    QColor color;
    float px2mmScreen;
    QSizeF sizeTouchpad;

    int createCursor(QPointF pos);

public:
    AbstractWidget(GhostView *g, QColor col);
    virtual ~AbstractWidget();

    void setName(QString n){ name = n; }
    QColor getColor() { return color; }
    void setParameters(float px2mmScreen, QSizeF sizeTouchpad);

    QMap <int, Cursor*> getCursors() { return cursors; }
    virtual QRectF boundingRect() const;
    virtual QPainterPath shape() const =0;
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);

    virtual void underPaint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) =0;
    virtual bool sceneEvent(QEvent *event) =0;

    float getWidth(){ return x_width; }
    void setWidth(float w){ x_width = w; }
    float getHeight(){ return y_height; }
    void setHeight(float h){ y_height = h; }

    void callNeedTuioRecalibrate(){ needTuioRecalibrate(this); }

    QList<QTouchEvent::TouchPoint> getTouchpointList();

signals:
    void receiveTouchBegin();
    void receiveTouchEnd();
    void needTuioRecalibrate(AbstractWidget* widget);
    void myDestroyed(QString name);

public slots:
    void frameRotate();
};

#endif // ABSTRACTWIDGET_H
