/*
 * GhostView - WidgetControl overlay for Qt
 *
 * Author: Jérémie Gilliot <jeremie.gilliot@inria.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GHOSTVIEW_H
#define GHOSTVIEW_H

#include <QGraphicsView>
#include <QWidget>
#include <QMap>
#include <QTouchEvent>
#include <QPixmap>

#include "TouchDispatcher.h"

class GhostView : public QGraphicsView, public TouchDispatcher
{
    Q_OBJECT
public:
    GhostView(QGraphicsScene *scene, QWidget *parent, QWidget *uA);

    QList<QTouchEvent::TouchPoint> getTouchpointList();
    QPixmap getPrintScreen();

    static float px2mmScreen;
    static float scaleFactor;
    static float circleSize;
    static void setPx2mmScreen(float px, float sf);

    void launchPrintScreenTimer();
    void stopPrintScreenTimer();

protected:
    bool viewportEvent(QEvent *event);

    void updateTouch(QTouchEvent::TouchPoint *newTouchPoint);
    struct touchWidget{
        QTouchEvent::TouchPoint tPoint;
        QWidget *widget;
    };
    QMap<int, struct touchWidget> touchPointList;
    QMap<QWidget *, QList<int> > widgets;
private:
    QWidget *underApp;
    QGraphicsView *underAppView;
    QGraphicsScene *underAppScene;

    QMap<int, QTouchEvent::TouchPoint> listPtsOnGhostView;

    QTimer *timerPrintScreen;
    QPixmap radar;

public slots:
    void updatePrintScreen();
};

#endif // GHOSTVIEW_H
