#include "mingetclose.h"

#include <QApplication>
#include <QGraphicsScene>
#include <QGraphicsSceneMouseEvent>
#include <QPainter>
#include <QStyleOption>
#include <QTouchEvent>

#include "ghostview.h"

#define ADJUST 10
#define BUTTON_SIZE (GhostView::scaleFactor*3/GhostView::px2mmScreen)

MingetClose::MingetClose(QGraphicsObject *cW) : QGraphicsObject(), controlWidget(cW)
{
    updatePos();
    connect(controlWidget, SIGNAL(heightChanged()), this, SLOT(updatePos()));
    connect(controlWidget, SIGNAL(widthChanged()), this, SLOT(updatePos()));
    setAcceptTouchEvents(true);
}

QRectF MingetClose::boundingRect() const
{
    return QRectF(-BUTTON_SIZE - ADJUST, -BUTTON_SIZE - ADJUST,
                  2*BUTTON_SIZE + 2*ADJUST, 2*BUTTON_SIZE + 2*ADJUST);
}

QPainterPath MingetClose::shape() const
{
    QPainterPath path;
    path.addEllipse(-BUTTON_SIZE, -BUTTON_SIZE, BUTTON_SIZE * 2, BUTTON_SIZE * 2);
    return path;
}

void MingetClose::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
{
    float thic = GhostView::scaleFactor*1/GhostView::px2mmScreen;
    QPainterPath path;
    path.setFillRule(Qt::WindingFill);
    path.addRect(-BUTTON_SIZE, -thic/2.0, 2*BUTTON_SIZE, thic);
    path.addRect(-thic/2.0, -BUTTON_SIZE, thic, 2*BUTTON_SIZE);
    painter->rotate(45);
    painter->fillPath(shape().subtracted(path), QColor(150, 50, 50));
}

bool MingetClose::sceneEvent(QEvent *event)
{
    QTouchEvent *touchEvent;

    switch (event->type()) {
    case QEvent::TouchBegin:
        return true;
    case QEvent::TouchUpdate:
        return true;
    case QEvent::TouchEnd:
        touchEvent = static_cast<QTouchEvent *>(event);
        if(!touchEvent->touchPoints().isEmpty() && contains(touchEvent->touchPoints().first().pos()))
            controlWidget->deleteLater();
        return true;

    default:
        return QGraphicsItem::sceneEvent(event);
    }
}

void MingetClose::updatePos(){
    setPos(controlWidget->boundingRect().width()/2.0-ADJUST-2*BUTTON_SIZE, -controlWidget->boundingRect().height()/2.0+ADJUST+2*BUTTON_SIZE);
}
