#include "mingeticonify.h"

#include <QApplication>
#include <QGraphicsScene>
#include <QGraphicsSceneMouseEvent>
#include <QPainter>
#include <QStyleOption>
#include <QTouchEvent>

#include "mingetframe.h"
#include "ghostview.h"

#define ADJUST 10
#define BUTTON_SIZE (GhostView::scaleFactor*3/GhostView::px2mmScreen)

MingetIconify::MingetIconify(MingetFrame *f) : QGraphicsObject(), frame(f)
{
    updatePos();
    connect(frame, SIGNAL(heightChanged()), this, SLOT(updatePos()));
    connect(frame, SIGNAL(widthChanged()), this, SLOT(updatePos()));
    setAcceptTouchEvents(true);
}

QRectF MingetIconify::boundingRect() const
{
    return QRectF(-BUTTON_SIZE - ADJUST, -BUTTON_SIZE - ADJUST,
                  2*BUTTON_SIZE + 2*ADJUST, 2*BUTTON_SIZE + 2*ADJUST);
}

QPainterPath MingetIconify::shape() const
{
    QPainterPath path;
    path.addEllipse(-BUTTON_SIZE, -BUTTON_SIZE, BUTTON_SIZE * 2, BUTTON_SIZE * 2);
    return path;
}

void MingetIconify::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
{
    float thic = (GhostView::scaleFactor*1/GhostView::px2mmScreen);
    float pt = GhostView::scaleFactor;
    QPainterPath path;
    path.addEllipse(QPoint(0, 0), thic/2, thic/2);
    path.addEllipse(QPoint(0, -(BUTTON_SIZE-thic)), pt, pt);
    path.addEllipse(QPoint(0.87*(BUTTON_SIZE-thic), -0.5*(BUTTON_SIZE-thic)), pt, pt);
    path.addEllipse(QPoint(0.87*(BUTTON_SIZE-thic), 0.5*(BUTTON_SIZE-thic)), pt, pt);
    path.addEllipse(QPoint(0, (BUTTON_SIZE-thic)), pt, pt);
    path.addEllipse(QPoint(-0.87*(BUTTON_SIZE-thic), 0.5*(BUTTON_SIZE-thic)), pt, pt);
    path.addEllipse(QPoint(-0.87*(BUTTON_SIZE-thic), -0.5*(BUTTON_SIZE-thic)), pt, pt);
    painter->fillPath(shape().subtracted(path), QColor(100, 100, 100));
}

bool MingetIconify::sceneEvent(QEvent *event)
{
    QTouchEvent *touchEvent;

    switch (event->type()) {
    case QEvent::TouchBegin:
        return true;
    case QEvent::TouchUpdate:
        return true;
    case QEvent::TouchEnd:
        touchEvent = static_cast<QTouchEvent *>(event);
        if(!touchEvent->touchPoints().isEmpty() && contains(touchEvent->touchPoints().first().pos()))
            frame->toggleIconify();
        return true;

    default:
        return QGraphicsItem::sceneEvent(event);
    }
}

void MingetIconify::updatePos(){
    if(frame->isIconify())
        setPos(frame->boundingRect().width()/2.0-ADJUST-3*BUTTON_SIZE, -frame->boundingRect().height()/2.0+ADJUST+3*BUTTON_SIZE);
    else
        setPos(frame->boundingRect().width()/2.0-ADJUST-2*BUTTON_SIZE, -frame->boundingRect().height()/2.0+ADJUST+7*BUTTON_SIZE);
}
